﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Windows.Forms;
using EnglishTranslate.UtilClass;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Text.RegularExpressions;


namespace EnglishTranslate
{
    public partial class TranslateForm : Form
    {
        //获取所有的语种
        Dictionary<string, string> languageTypes = null;
        //获取Appid 和 SecretKey
        Dictionary<string, string> configJsons = null;
        //记录最后一句话
        private StringBuilder strDayWord = null;
        public TranslateForm()
        {
            InitializeComponent();
            this.RichTxtSrc.DragEnter += new System.Windows.Forms.DragEventHandler(this.RichTxtSrc_DragEnter);
            this.RichTxtSrc.DragDrop += new System.Windows.Forms.DragEventHandler(this.RichTxtSrc_DragDrop);

        }
        //翻译
        private void Button1_Click(object sender, EventArgs e)
        {
            this.btnClearTxt.Enabled = false;
            this.btnCopyToClipboard.Enabled = false;
            this.BtnOcrOpenImg.Enabled = false;
            if (languageTypes == null)
            {
                return;
            }
            //若没有原文 直接返回
            if (this.RichTxtSrc.Text == "")
            {
                return;
            }
            try
            {
                string from = "";
                string to = "";
                //原文去换行符
                string strSrc = this.RichTxtSrc.Text.Replace("\n", "");

               //当为自动检测模式时 中文翻译为英文 英文翻译为中文
                if(this.comboBoxDst.SelectedItem.ToString() == "自动检测")
                {
                    Regex regChina = new Regex("^[^\x00-\xFF]");
                    Regex regEnglish = new Regex("^[a-zA-Z]");
                    if (regEnglish.IsMatch(strSrc))
                    {
                        to = languageTypes["中文"].ToString();
                        
                    }
                    else if (regChina.IsMatch(strSrc))
                    {
                        to = languageTypes["英语"].ToString();
                    }
                }
                else
                {
                    to = languageTypes[this.comboBoxDst.SelectedItem.ToString()];
                }       
                from = languageTypes[this.comboBoxSrc.SelectedItem.ToString()];
                //翻译
                string result = UtilTraslate.translateWords(strSrc, from, to, configJsons["AppID"], configJsons["secretKey"]);
                JObject jo = (JObject)JsonConvert.DeserializeObject(result);
                RichTxtDst.Text = jo["trans_result"][0]["dst"].ToString();
            }
            catch
            {
                MessageBox.Show("翻译失败");
            }
            finally
            {
                this.btnClearTxt.Enabled = true;
                this.btnCopyToClipboard.Enabled = true;
                this.BtnOcrOpenImg.Enabled = true;
            }
        }

        #region panel初始化 加上边框
        // panel初始化加边框
        private void Panel1_Paint(object sender, PaintEventArgs e)
        {
            ControlPaint.DrawBorder(e.Graphics,
                                       panelSrc.ClientRectangle,
                                       Color.LightGray,
                                       1,
                                       ButtonBorderStyle.Solid,
                                       Color.LightGray,
                                       1,
                                       ButtonBorderStyle.Solid,
                                       Color.LightGray,
                                       1,
                                       ButtonBorderStyle.Solid,
                                       Color.LightGray,
                                       1,
                                       ButtonBorderStyle.Solid);
        }
        // panel初始化加边框
        private void PanelDst_Paint(object sender, PaintEventArgs e)
        {
            ControlPaint.DrawBorder(e.Graphics,
                                      panelSrc.ClientRectangle,
                                      Color.LightGray,
                                      1,
                                      ButtonBorderStyle.Solid,
                                      Color.LightGray,
                                      1,
                                      ButtonBorderStyle.Solid,
                                      Color.LightGray,
                                      1,
                                      ButtonBorderStyle.Solid,
                                      Color.LightGray,
                                      1,
                                      ButtonBorderStyle.Solid);
        }
        #endregion


        #region 窗体拖动
        //拖动的形式
        [DllImport("user32.DLL", EntryPoint = "ReleaseCapture")]
        private extern static void ReleaseCapture();

        [DllImport("user32.DLL", EntryPoint = "SendMessage")]
        private extern static void SendMessage(System.IntPtr hWnd, int wMsg, int wParam, int lParam);
        // 鼠标点中此区域可以拖动
        private void PanelHeader_MouseDown(object sender, MouseEventArgs e)
        {
            ReleaseCapture();
            SendMessage(this.Handle, 0x112, 0xf012, 0);
        }
        #endregion


        #region 关闭 最大化 最小化 窗体
        // 关闭窗体
        private void BtnClose_Click(object sender, EventArgs e)
        {
            timer.Start();
            //Application.Exit();
        }
        //最大化
        private void BtnMax_Click(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Normal)
            {
                WindowState = FormWindowState.Maximized;
            }
            else
            {
                WindowState = FormWindowState.Normal;
            }
        }
        //最小化
        private void BtnMuim_Click(object sender, EventArgs e)
        {
            WindowState = FormWindowState.Minimized;
        }
        #endregion

        // 窗体初始化
        private void TranslateForm_Load(object sender, EventArgs e)
        {
            //初时化语种
            languageTypes = GetLanguageType();
            foreach (var item in languageTypes)
            {
                comboBoxDst.Items.Add(item.Key);
                comboBoxSrc.Items.Add(item.Key);
            }
            //默认原文为自动检测 译文为自动检测
            comboBoxDst.SelectedIndex = 0;
            comboBoxSrc.SelectedIndex = 0;
            configJsons = UtilJsonReaderAndWriter.ReaderJson(UtilFinalPath.SettingPath);

            this.RichTxtSrc.SelectionColor = Color.LightGray;

            
        }
        /// <summary>
        /// 获取所有的语种
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetLanguageType()
        {
            Dictionary<string, string> dicLanguageTypes = new Dictionary<string, string>();
            string strLanguageTypes = Properties.Resources.language;
            //依据文本结构 按 = 分成字符串数组
            string[] arrayLanguageTypes = strLanguageTypes.Trim().Split('=');
            for (int i = 0; i < arrayLanguageTypes.Length; i += 2)
            {
                dicLanguageTypes.Add(arrayLanguageTypes[i + 1].Replace("\r\n",""), arrayLanguageTypes[i]);
            }
            return dicLanguageTypes;
        }

        //清空原文
        private void Button4_Click(object sender, EventArgs e)
        {
            this.RichTxtSrc.Text = "";
            this.RichTxtDst.Text = "";
        }
        //复制译文
        private void Button6_Click(object sender, EventArgs e)
        {
            if (this.RichTxtDst.Text != "")
            {
                Clipboard.SetDataObject(this.RichTxtDst.Text);
            }
        }

        #region 任务栏右键显示的菜单
        private void 显示ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            WindowState = FormWindowState.Normal;
        }
        private void 退出ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }


        private void 设置ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Settings().ShowDialog();
            configJsons = UtilJsonReaderAndWriter.ReaderJson(UtilFinalPath.SettingPath);
        }
        #endregion

        #region 关闭 最大化 最小化 鼠标经过时按钮颜色的变化
        //关闭按钮图标颜色
        private void BtnClose_MouseMove(object sender, MouseEventArgs e)
        {
            BtnClose.BackColor = Color.Red;
        }

        private void BtnClose_MouseLeave(object sender, EventArgs e)
        {

            BtnClose.BackColor = Color.FromArgb(26, 26, 26);
        }
        //最大化按钮图标颜色
        private void BtnMax_MouseMove(object sender, MouseEventArgs e)
        {
            BtnMax.BackColor = Color.FromArgb(46, 46, 46);
        }

        private void BtnMax_MouseLeave(object sender, EventArgs e)
        {
            BtnMax.BackColor = Color.FromArgb(26, 26, 26);
        }
        //最小化按钮图标颜色
        private void BtnMuim_MouseMove(object sender, MouseEventArgs e)
        {
            BtnMuim.BackColor = Color.FromArgb(46, 46, 46);
        }

        private void BtnMuim_MouseLeave(object sender, EventArgs e)
        {
            BtnMuim.BackColor = Color.FromArgb(26, 26, 26);
        }
        #endregion

        #region 快捷键 ctrl + enter
        private void TranslateForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter && e.Modifiers == Keys.Control)
            {
                Button1_Click(this, EventArgs.Empty);
            }
        }
        #endregion

        #region 悬浮按钮时的文字提示  
    //清除文本提示文字
    private void Button4_MouseHover(object sender, EventArgs e)
        {
            SuspensionControl(this.btnClearTxt, "清除文本内容", 5000, 500, 500, true);

        }
        //复制译文提示文字
        private void BtnCopyToClipboard_MouseHover(object sender, EventArgs e)
        {
            SuspensionControl(this.btnCopyToClipboard, "复制译文到粘贴板", 5000, 500, 500, true);

        }

        /// <summary>
        /// 鼠标悬浮在空间上时显示的文字
        /// </summary>
        /// <param name":control">要悬浮的控件</param>
        /// <param name":suspensionInfo">提示信息</param>
        /// <param name":autoPopDelay">提示信息的可见时间</param>
        /// <param name":initialDelay">事件触发多久后出现提示</param>
        /// <param name":reshowDelay">指针从一个控件移向另一个控件时，经过多久才会显示下一个提示框</param>
        /// <param name":showAlways">/是否显示提示框</param>
        private void SuspensionControl(Control control, string suspensionInfo, int autoPopDelay, int initialDelay, int reshowDelay, bool showAlways)
        {
            // 创建the ToolTip
            ToolTip toolTip1 = new ToolTip();

            // 设置显示样式
            toolTip1.AutoPopDelay = autoPopDelay;//提示信息的可见时间
            toolTip1.InitialDelay = initialDelay;//事件触发多久后出现提示
            toolTip1.ReshowDelay = reshowDelay;//指针从一个控件移向另一个控件时，经过多久才会显示下一个提示框
            toolTip1.ShowAlways = showAlways;//是否显示提示框

            //  设置伴随的对象.
            toolTip1.SetToolTip(control, suspensionInfo);//设置提示按钮和提示内容
        }
        #endregion

        private void BtnSetting_Click(object sender, EventArgs e)
        {
            new Settings().ShowDialog();
            configJsons = UtilJsonReaderAndWriter.ReaderJson(UtilFinalPath.SettingPath);
        }
    
       
       

        private async void OcrDistinguish(string ImagePath,string imageExtension)
        {
            this.btnClearTxt.Enabled = false;
            this.btnCopyToClipboard.Enabled = false;
            this.btnTranslate.Enabled = false;
            this.BtnOcrOpenImg.Enabled = false;
            string content = "";
            try
            {
                HttpClient httpClient = new HttpClient();
                httpClient.Timeout = new TimeSpan(1, 1, 1);

                //表单
                MultipartFormDataContent form = new MultipartFormDataContent();
                //在表单数据中添加了api密钥 helloworld
                form.Add(new StringContent(configJsons["OCRKey"]), "apikey"); //Added api key in form data
                //在配置文件中读取语种编号
                form.Add(new StringContent(UtilOcr.getSelectedLanguage(Convert.ToInt32(configJsons["OCRLanguage"]))), "language");
                if (string.IsNullOrEmpty(ImagePath) == false)
                {
                    //将图片转为 字节数组
                    byte[] imageData = File.ReadAllBytes(ImagePath);
                    form.Add(new ByteArrayContent(imageData, 0, imageData.Length), "image", "image"+ imageExtension);
                }
                //else if (string.IsNullOrEmpty(PdfPath) == false)
                //{
                //    byte[] imageData = File.ReadAllBytes(PdfPath);
                //    form.Add(new ByteArrayContent(imageData, 0, imageData.Length), "PDF", "pdf.pdf");
                //}

                HttpResponseMessage response = await httpClient.PostAsync("https://api.ocr.space/Parse/Image", form);

                string strContent = await response.Content.ReadAsStringAsync();



                OCRResult.Rootobject ocrResult = JsonConvert.DeserializeObject<OCRResult.Rootobject>(strContent);


                if (ocrResult.OCRExitCode == 1)
                {
                    for (int i = 0; i < ocrResult.ParsedResults.Count(); i++)
                    {
                        content = content + ocrResult.ParsedResults[i].ParsedText;
                    }
                }
                else
                {
                    Console.WriteLine("ERROR: " + "," + strContent);
                }

                //Console.WriteLine(content);
                this.RichTxtDst.Text = content;               
            }
            catch (Exception exception)
            {
                Console.WriteLine("Ooops" + "," + exception.Message);

            }
            finally
            {
                this.btnClearTxt.Enabled = true;
                this.btnCopyToClipboard.Enabled = true;
                this.btnTranslate.Enabled = true;
                this.BtnOcrOpenImg.Enabled = true;
            }
        }

        private void BtnScreenShot_Click(object sender, EventArgs e)
        {          
            OpenFileDialog fileDlg = new OpenFileDialog();
            fileDlg.Filter = "image files|*.jpg;*.JPG;*.gif;*.GIF;*.png;*.PNG;*.tif;*.TIF;*.bmp;*.bmp";
            if (fileDlg.ShowDialog() == DialogResult.OK)
            {
                FileInfo fileInfo = new FileInfo(fileDlg.FileName);
                if (fileInfo.Length > 5 * 1024 * 1024)
                {
                    //Size limit depends: Free API 1 MB, PRO API 5 MB and more
                    MessageBox.Show("图片大小要求在1M以内");
                    return;
                }
                Clipboard.Clear();   //清空剪贴板
                Bitmap bmp = new Bitmap(fileDlg.FileName);  //创建Bitmap类对象
                Clipboard.SetImage(bmp);  //将Bitmap类对象写入剪贴板
                this.RichTxtSrc.Paste();   //将剪贴板中的对象粘贴到RichTextBox1
                //this.RichTxtSrc.Paste()
                //识别
                string imageExtension = Path.GetExtension(fileDlg.FileName).ToLower();
                OcrDistinguish(fileDlg.FileName, imageExtension);                            
            }
          
        }

       

        /// <summary>
        /// 富文本框拖动文件 时
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RichTxtSrc_DragEnter(object sender,System.Windows.Forms.DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.Move;
            else
                e.Effect = DragDropEffects.None;
        }

        private void RichTxtSrc_DragDrop(object sender,System.Windows.Forms.DragEventArgs e)
        {     
            string[] paths = (string[])e.Data.GetData(DataFormats.FileDrop);
            string imageExtension = Path.GetExtension(paths[0]).ToLower();
            OcrDistinguish(paths[0],imageExtension);           
        }
        //显示提示文字
        private void BtnOcrOpenImg_MouseHover(object sender, EventArgs e)
        {
            SuspensionControl(this.BtnOcrOpenImg, "选择要识别的图片", 5000, 500, 500, true);
        }
        private void BtnClearTxt_Click(object sender, EventArgs e)
        {
            this.RichTxtSrc.Clear();
            this.RichTxtDst.Clear();
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            if (this.Opacity >= 0.025)
            {
                this.Opacity -= 0.025;
            }
            else
            {
                timer.Stop();
                Application.Exit();
            }
        }
    }
}
